"""
The :mod:`pai4sk.decomposition` module includes matrix decomposition
algorithms, including among others PCASklearn, NMF or ICA. Most of the algorithms of
this module can be regarded as dimensionality reduction techniques.
"""

from .nmf import NMF, non_negative_factorization
from .pca import PCASklearn
from .incremental_pca import IncrementalPCA
from .kernel_pca import KernelPCA
from .sparse_pca import SparsePCA, MiniBatchSparsePCA
from .truncated_svd import TruncatedSVDSklearn
from .fastica_ import FastICA, fastica
from .dict_learning import (dict_learning, dict_learning_online, sparse_encode,
                            DictionaryLearning, MiniBatchDictionaryLearning,
                            SparseCoder)
from .factor_analysis import FactorAnalysis
from ..utils.extmath import randomized_svd
from .online_lda import LatentDirichletAllocation

__all__ = ['DictionaryLearning',
           'FastICA',
           'IncrementalPCA',
           'KernelPCA',
           'MiniBatchDictionaryLearning',
           'MiniBatchSparsePCA',
           'NMF',
           'PCASklearn',
           'SparseCoder',
           'SparsePCA',
           'dict_learning',
           'dict_learning_online',
           'fastica',
           'non_negative_factorization',
           'randomized_svd',
           'sparse_encode',
           'FactorAnalysis',
           'TruncatedSVDSklearn',
           'LatentDirichletAllocation']
import inspect
if '_cuml/solvers/pca' not  in  [item[1]  for item in inspect.stack() if '<frozen importlib._bootstrap' not  in str(item[1]) ] [1] : from .._cuml.solvers.pca import PCA
if '_cuml/solvers/truncated_svd' not  in  [item[1]  for item in inspect.stack() if '<frozen importlib._bootstrap' not  in str(item[1]) ] [1] : from .._cuml.solvers.truncated_svd import TruncatedSVD
