# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from scipy import sparse

# Load the data
import time
import numpy as np
import argparse

defaultPath = "."
CLI=argparse.ArgumentParser()

CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)
CLI.add_argument(
    "--num_threads",
    type=int,
    default=8
)
args = CLI.parse_args()

t0 = time.time()
X_train = np.load(args.data_path + "/data/HIGGS.X_train.npy")
X_test  = np.load(args.data_path + "/data/HIGGS.X_test.npy")
y_train = np.load(args.data_path + "/data/HIGGS.y_train.npy")
y_test  = np.load(args.data_path + "/data/HIGGS.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

max_depth = None
n_ex = X_train.shape[0]
n_estimators = 10
max_features = 5
n_jobs = args.num_threads

# Import snap RandomForestClassifier from pai4sk module directly
from pai4sk import RandomForestClassifier as SnapForest
dt = SnapForest(random_state=0, verbose=False, max_depth=max_depth, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)
        
# Training time
t0 = time.time()
dt.fit(X_train[0:n_ex], y_train[0:n_ex])
print("[snap] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred_test = dt.predict(X_test)

from sklearn.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred_test)
print("[snap] Accuracy score:   {0:.4f}".format(acc_snap))
        
if not args.skip_sklearn:
    # Import sklearn's DecisionTreeClassifier from pai4sk module directly
    from pai4sk.ensemble import RandomForestClassifier as SklearnForest
    
    dt = SklearnForest(random_state=0, max_depth=max_depth, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)
    
    # Training time
    t0 = time.time()
    dt.fit(X_train[0:n_ex], y_train[0:n_ex])
    print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = dt.predict(X_test)

    # Evaluate accuracy_score on test set
    from sklearn.metrics import accuracy_score
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn] Accuracy score:   {0:.4f}".format(acc_sklearn))
